// (C) 2007-2025 GoodData Corporation
import isEmpty from "lodash/isEmpty.js";
import { v4 as uuid } from "uuid";
import { decoratedBackend } from "../decoratedBackend/index.js";
import { DecoratedExecutionFactory, DecoratedExecutionResult, DecoratedPreparedExecution, } from "../decoratedBackend/execution.js";
class WithExecutionEventing extends DecoratedPreparedExecution {
    callbacks;
    constructor(decorated, callbacks, signal) {
        super(decorated, signal);
        this.callbacks = callbacks;
    }
    execute = () => {
        const { beforeExecute, successfulExecute, failedExecute } = this.callbacks;
        const executionId = uuid();
        beforeExecute?.(this.definition, executionId);
        return super
            .execute()
            .then((result) => {
            successfulExecute?.(result, executionId);
            return new WithExecutionResultEventing(result, this.createNew, this.callbacks, executionId);
        })
            .catch((error) => {
            failedExecute?.(error, executionId);
            throw error;
        });
    };
    createNew = (decorated, signal) => {
        return new WithExecutionEventing(decorated, this.callbacks, signal);
    };
}
class WithExecutionResultEventing extends DecoratedExecutionResult {
    callbacks;
    executionId;
    constructor(decorated, wrapper, callbacks, executionId) {
        super(decorated, wrapper);
        this.callbacks = callbacks;
        this.executionId = executionId;
    }
    readAll = () => {
        const { successfulResultReadAll, failedResultReadAll } = this.callbacks;
        const promisedDataView = super.readAll();
        return promisedDataView
            .then((res) => {
            successfulResultReadAll?.(res, this.executionId);
            return res;
        })
            .catch((e) => {
            failedResultReadAll?.(e, this.executionId);
            throw e;
        });
    };
    readForecastAll(config) {
        const { successfulForecastResultReadAll, failedForecastResultReadAll } = this.callbacks;
        const promisedDataView = super.readForecastAll(config);
        return promisedDataView
            .then((res) => {
            successfulForecastResultReadAll?.(res, this.executionId);
            return res;
        })
            .catch((e) => {
            failedForecastResultReadAll?.(e, this.executionId);
            throw e;
        });
    }
    readAnomalyDetectionAll(config) {
        return super.readAnomalyDetectionAll(config);
    }
    readClusteringAll(config) {
        return super.readClusteringAll(config);
    }
    readWindow = (offset, size) => {
        const { successfulResultReadWindow, failedResultReadWindow } = this.callbacks;
        const promisedDataView = super.readWindow(offset, size);
        return promisedDataView
            .then((res) => {
            successfulResultReadWindow?.(offset, size, res, this.executionId);
            return res;
        })
            .catch((e) => {
            failedResultReadWindow?.(offset, size, e, this.executionId);
            throw e;
        });
    };
}
/**
 * This implementation of analytical backend decorates another implementation with support for eventing. Events
 * for significant actions are emitted to defined callback functions (event handlers).
 *
 * @param realBackend - backend supplying the actual implementation of SPI
 * @param callbacks - callbacks to event handlers
 * @beta
 */
export function withEventing(realBackend, callbacks) {
    if (isEmpty(callbacks)) {
        return realBackend;
    }
    return decoratedBackend(realBackend, {
        execution: (original) => new DecoratedExecutionFactory(original, (execution) => new WithExecutionEventing(execution, callbacks, execution.signal)),
    });
}
//# sourceMappingURL=index.js.map